class TC_Files < Test::Unit::TestCase

    include P4RubyTest

    def name
	"Test file operations"
    end

    def test_add_files
	assert( p4.connect, "Failed to connect to Perforce server" )
	assert( p4.run_opened.length == 0, "Shouldn't have any open files" )
	Dir.mkdir( "test_files" )
	%w{ foo bar baz }.each do
	    |fn|
	    fn = "test_files/#{fn}.txt"
	    File.open( fn, "w" ) do
		|f|
		f.puts( "This is a test file" )
	    end
	    p4.run_add( fn )
	end
	assert( p4.run_opened.length == 3, "Unexpected number of open files" )

	change = p4.fetch_change
	assert( change.kind_of?( P4::Spec ), "Change form is not a spec" )
	change._description = "Add some test files\n"
	assert( change._description == "Add some test files\n",
	        "Change description not set properly" )
	assert_submit( "Failed to add files", change )

	# Ensure no files are open and that all files are present
	assert( p4.run_opened.length == 0 )
	assert( p4.run_files( 'test_files/...' ).length == 3 )

	# Now edit the files, and submit another revision.
	assert( p4.run_edit( 'test_files/...' ).length == 3 )
	change = p4.fetch_change
	change._description = "Editing the test files"
	assert_submit( "Failed to submit edits", change )
	assert( p4.run_opened.length == 0 )

	# Now branch the files
	assert( p4.run_integ( 'test_files/...', 'test_branch/...' ) )
	change = p4.fetch_change
	change._description = "Branching the test files"
	assert_submit( "Failed to submit branch1", change )
	assert( p4.run_opened.length == 0 )

	# And now branch them again
	assert( p4.run_integ( 'test_files/...', 'test_branch2/...' ) )
	change = p4.fetch_change
	change._description = "Branching the test files again"
	assert_submit( "Failed to submit branch2", change )
	assert( p4.run_opened.length == 0 )

	# Now check out 'p4 filelog'
	files = p4.run_filelog( 'test_files/...' )
	assert( files.length == 3 )

	df = files[ 0 ]
	assert( df.depot_file == "//depot/test_files/bar.txt", df.depot_file )
	assert( df.revisions.length == 2 )
	
	rev = df.revisions[ 0 ]
	assert( rev.rev == 2 )
	assert( rev.integrations.length == 2 )
	assert( rev.integrations[ 0 ].how == "branch into" )
	assert( rev.integrations[ 0 ].file == "//depot/test_branch/bar.txt" )

    	p4.disconnect
    end
    
    #
    # Local method to help ensure submits are working
    #
    def assert_submit( msg, *args )
	assert_block( msg ) do
	    begin
		result = @p4.run_submit( args )
		if( result[-1].has_key?( 'submittedChange' ) )
		    true
		else
		    false
		end
	    rescue P4Exception
		false
	    end
	end
    end

end
